USE master;
GO

-- Set up a login for the test user
CREATE LOGIN MyCommonLogin
   WITH PASSWORD = 'ASDECd2439587y'
GO

USE [TEST];
GO

CREATE USER MyCommonUser
FOR LOGIN MyCommonLogin;
GO

-- Create a stored procedure
CREATE PROCEDURE dbo.MyTestSP
AS
BEGIN
   -- Shows who is running the stored procedure
   SELECT SYSTEM_USER 'system Login'
   , USER AS 'Database Login'
   , NAME AS 'Context'
   , TYPE
   , USAGE 
   FROM sys.user_token;   

   -- Now get the data
   SELECT * FROM sys.dm_exec_connections;
END
GO

GRANT EXECUTE ON [dbo].[MyTestSP] TO [MyCommonUser];
GO

-- Create a certificate
USE [master];
GO

CREATE CERTIFICATE MyCertificate
   ENCRYPTION BY PASSWORD = 'pGFD4bb925DGvbd2439587y'
      WITH SUBJECT = 'View Connections Access', 
      EXPIRY_DATE = '12/05/2015';
GO

CREATE LOGIN MyPrivilidgedLogin FROM CERTIFICATE MyCertificate
GO

GRANT VIEW SERVER STATE TO MyPrivilidgedLogin
GO

BACKUP CERTIFICATE MyCertificate TO FILE = 'C:\temp\MyCertificate.cer'
WITH PRIVATE KEY (FILE = 'C:\temp\MyCertificate.pvk' ,
                  ENCRYPTION BY PASSWORD = 'pGFD4bb925DGvbd2439587y',
                  DECRYPTION BY PASSWORD = 'pGFD4bb925DGvbd2439587y');
GO

USE [TEST];
GO

CREATE CERTIFICATE MyCertificate FROM FILE = 'C:\temp\MyCertificate.cer'
WITH PRIVATE KEY (FILE = 'C:\temp\MyCertificate.pvk',
                  DECRYPTION BY PASSWORD = 'pGFD4bb925DGvbd2439587y',
                  ENCRYPTION BY PASSWORD = 'pGFD4bb925DGvbd2439587y')
GO

EXEC xp_cmdshell 'DEL C:\temp\MyCertificate.*'
GO

CREATE USER MyPrivilidgedUser FOR CERTIFICATE MyCertificate
GO

GRANT EXECUTE ON dbo.MyTestSP TO MyPrivilidgedUser
go

-- Sign the test procedure each time you have changed it.
ADD SIGNATURE TO MyTestSP BY CERTIFICATE MyCertificate
    WITH PASSWORD = 'pGFD4bb925DGvbd2439587y'
GO


-- Clean up the example
USE [TEST];
GO

DROP PROCEDURE MyTestSP;
DROP USER MyPrivilidgedUser;
DROP USER MyCommonUser;
DROP CERTIFICATE MyCertificate;
GO

USE [master];
GO

DROP LOGIN MyCommonLogin;
DROP LOGIN MyPrivilidgedLogin;
DROP CERTIFICATE MyCertificate;
GO

