use TEST;
go

-- Generate some data
if object_id('dbo.MyBigTable', 'U') is not null
	drop table dbo.MyBigTable;
go

create table dbo.MyBigTable (
	keyval int not null primary key, 
	dataval int not null check (dataval between 1 and 10)
);
go

-- Insert 100000 rows
with digits
as (select d from (values (0),(1),(2),(3),(4),(5),(6),(7),(8),(9)) as d(d)) 
insert into dbo.mybigtable(keyval, dataval) 
select 10000 * tt.d + 1000 * st.d 
     + 100 * h.d + 10 * t.d + s.d + 1, 
       10 * rand(checksum(newid())) + 1 
from   digits as s,  digits as t,  digits as h, 
       digits as st, digits as tt;
go

-- Create my test function
if OBJECT_ID('dbo.fn_addone', N'FN') is not null
	drop function dbo.fn_addone;
go

create function dbo.fn_addone(@in int)
returns int
as
begin
  return @in + 1;
end
go

-- Include actual query plan: Ctrl+M

-- First test
select dbo.fn_addone(dataval) as val
from   dbo.MyBigTable;

select dataval + 1 as val
from   dbo.MyBigTable;

-- Second run
set statistics time on

select dbo.fn_addone(dataval) as val
from   dbo.MyBigTable;

select dataval + 1 as val
from   dbo.MyBigTable;

-- Actual plan is lying, let's see estimated plan
select dbo.fn_addone(dataval) as val
from   dbo.MyBigTable;

-- Determinism
select min(dbo.fn_addone(dataval)) as val
from dbo.MyBigTable;

select min(dataval + 1) as val
from dbo.MyBigTable;

-- IsDeterministic = 0
select OBJECTPROPERTYEX(object_id('dbo.fn_addone'), 'IsDeterministic') as IsDeterministic

select min(dbo.fn_addone(dataval)) as val
from (select distinct dataval from dbo.MyBigTable) as t;

select min(t.dataval + 1) as val
from (select distinct dataval from dbo.MyBigTable) as t;
go

-- Change function
alter function dbo.fn_addone(@in int)
returns int
with schemabinding
as
begin
  return @in + 1;
end
go

-- IsDeterministic = 1
select OBJECTPROPERTYEX(object_id('dbo.fn_addone'), 'IsDeterministic') as IsDeterministic

-- Once again
select min(dbo.fn_addone(dataval)) as val
from dbo.MyBigTable;

select min(dataval + 1) as val
from dbo.MyBigTable;

-- We cannot influence the cost


-- Parallelism killers
/*
create table dbo.MyBigTable2 (
	keyval int not null primary key, 
	dataval int not null check (dataval between 1 and 10)
);
go

with digits 
as (select d from (values (0),(1),(2),(3),(4),(5),(6),(7),(8),(9)) as d(d)) 
insert into dbo.MyBigTable2(keyval, dataval) 
select 1000000 * sm.d
     + 100000 * ht.d + 10000 * tt.d + 1000 * st.d
     + 100 * h.d + 10 * t.d + s.d + 1,
       10 * rand(checksum(newid())) + 1
from   digits as s,  digits as t,  digits as h,
       digits as st, digits as tt, digits as ht,
       digits as sm;
go
*/

select min(dbo.fn_addone(dataval)) as val
from   dbo.MyBigTable2;

select min(dataval + 1) as val
from   dbo.MyBigTable2;
go

/*
   CPU time = 46598 ms,  elapsed time = 50298 ms.
   CPU time = 3151 ms,  elapsed time = 2534 ms.
*/

-- And we haven't touched data yet







-- Inline functions
if OBJECT_ID('dbo.fn_addone_inline', N'IF') is not null
	drop function dbo.fn_addone_inline;
go

create function dbo.fn_addone_inline
(
	@i int
)
returns table as return
(
    select @i + 1 as out
);
go


set statistics time on

select dbo.fn_addone(dataval) as val
from   dbo.MyBigTable;

select dataval + 1 as val
from   dbo.MyBigTable;

select f.out as val
from   dbo.MyBigTable
cross apply dbo.fn_addone_inline(dataval) f;
